@@ -7,8 +7,8 @@
 #ifndef _filter_xbrz_h_
 #define _filter_xbrz_h_
 
-void filter_2xBRZ (uint8 *, uint32, uint8 *, uint32, int, int);
-void filter_3xBRZ (uint8 *, uint32, uint8 *, uint32, int, int);
-void filter_4xBRZ (uint8 *, uint32, uint8 *, uint32, int, int);
+void filter_2xBRZ (uint8 *, int, uint8 *, int, int, int);
+void filter_3xBRZ (uint8 *, int, uint8 *, int, int, int);
+void filter_4xBRZ (uint8 *, int, uint8 *, int, int, int);
 
 #endif
