@@ -5,8 +5,10 @@
 \*****************************************************************************/
 
 #include <fcntl.h>
+#include <filesystem>
 
 #include "SDL_joystick.h"
+#include "fscompat.h"
 #include "gtk_s9x.h"
 #include "gtk_config.h"
 #include "gtk_control.h"
@@ -62,7 +64,7 @@ const BindingLink b_links[] =
         { "b_bg_layer_2",          "ToggleBG2"         },
         { "b_bg_layer_3",          "ToggleBG3"         },
         { "b_sprites",             "ToggleSprites"     },
-        { "b_bg_layering_hack",    "BGLayeringHack"    },
+        { "toggle_backdrop",       "ToggleBackdrop"    },
         { "b_screenshot",          "Screenshot"        },
         { "b_fullscreen",          "GTK_fullscreen"    },
         { "b_state_save_current",  "GTK_state_save_current" },
@@ -188,8 +190,6 @@ static void swap_controllers_1_2()
 
 static void change_slot(int difference)
 {
-    static char buf[256];
-
     gui_config->current_save_slot += difference;
     gui_config->current_save_slot %= 1000;
     if (gui_config->current_save_slot < 0)
@@ -197,8 +197,16 @@ static void change_slot(int difference)
     if (!gui_config->rom_loaded)
         return;
 
-    snprintf(buf, 256, "State Slot: %d", gui_config->current_save_slot);
-    S9xSetInfoString(buf);
+    char extension_string[5];
+    snprintf(extension_string, 5, ".%03d", gui_config->current_save_slot);
+    auto filename = S9xGetFilename(extension_string, SNAPSHOT_DIR);
+    struct stat info;
+    std::string exists = "empty";
+    if (stat(filename.c_str(), &info) == 0)
+        exists = "used";
+
+    auto info_string = "State Slot: " + std::to_string(gui_config->current_save_slot) + " [" + exists + "]";
+    S9xSetInfoString(info_string.c_str());
     GFX.InfoStringTimeout = 60;
 }
 
