@@ -10,19 +10,20 @@
 #include "gtk_s9x.h"
 #include "gtk_display_driver.h"
 
-#include <epoxy/gl.h>
+#include <glad/gl.h>
 
-#include "gtk_opengl_context.h"
+#include "common/video/opengl_context.hpp"
 
 #include "gtk_compat.h"
 #ifdef GDK_WINDOWING_X11
-#include "gtk_glx_context.h"
+#include "common/video/glx_context.hpp"
 #endif
 #ifdef GDK_WINDOWING_WAYLAND
-#include "gtk_wayland_egl_context.h"
+#include "common/video/wayland_egl_context.hpp"
 #endif
 
 #include "shaders/glsl.h"
+#include "vulkan/std_chrono_throttle.hpp"
 
 #define BUFFER_OFFSET(i) ((char *)(i))
 
@@ -30,14 +31,17 @@ class S9xOpenGLDisplayDriver : public S9xDisplayDriver
 {
   public:
     S9xOpenGLDisplayDriver(Snes9xWindow *window, Snes9xConfig *config);
-    void refresh(int width, int height);
-    int init();
-    void deinit();
-    void update(uint16_t *buffer, int width, int height, int stride_in_pixels);
-    void *get_parameters();
-    void save(const char *filename);
+    void refresh() override;
+    int init() override;
+    void deinit() override;
+    void update(uint16_t *buffer, int width, int height, int stride_in_pixels) override;
+    void *get_parameters() override;
+    void save(const char *filename) override;
     static int query_availability();
-    bool is_ready();
+    bool is_ready() override;
+    bool can_throttle() override { return true; }
+    int get_width() final override { return output_window_width; }
+    int get_height() final override { return output_window_height; }
 
   private:
     bool opengl_defaults();
@@ -52,13 +56,10 @@ class S9xOpenGLDisplayDriver : public S9xDisplayDriver
     GLint texture_width;
     GLint texture_height;
     GLuint texmap;
-    GLuint pbo;
 
     bool legacy;
     bool core;
     int version;
-    bool npot;
-    bool using_pbos;
     bool initialized;
 
     bool using_glsl_shaders;
@@ -70,6 +71,8 @@ class S9xOpenGLDisplayDriver : public S9xDisplayDriver
 
     OpenGLContext *context;
 
+    Throttle throttle;
+
 #ifdef GDK_WINDOWING_X11
     GTKGLXContext glx;
 #endif
