@@ -4,10 +4,9 @@
    For further information, consult the LICENSE file in the root directory.
 \*****************************************************************************/
 
-#include <stdio.h>
 #include <signal.h>
-#include "giomm/application.h"
-#include "glibmm/main.h"
+#define G_LOG_USE_STRUCTURED
+#define G_LOG_DOMAIN GETTEXT_PACKAGE
 #include "gtk_compat.h"
 #include "gtk_config.h"
 #include "gtk_s9x.h"
@@ -27,6 +26,9 @@
 #include "gfx.h"
 #include "memmap.h"
 #include "ppu.h"
+#include "fmt/format.h"
+
+#include <iomanip>
 
 static void S9xThrottle(int);
 static void S9xCheckPointerTimer();
@@ -34,8 +36,8 @@ static bool S9xIdleFunc();
 static bool S9xPauseFunc();
 static bool S9xScreenSaverCheckFunc();
 
-Snes9xWindow *top_level;
-Snes9xConfig *gui_config;
+Snes9xWindow *top_level = nullptr;
+Snes9xConfig *gui_config = nullptr;
 StateManager state_manager;
 gint64 frame_clock = -1;
 gint64 pointer_timestamp = -1;
@@ -49,15 +51,20 @@ static void S9xTerm(int signal)
 
 int main(int argc, char *argv[])
 {
-    struct sigaction sig_callback;
     auto app = Gtk::Application::create("com.snes9x.gtk", Gio::APPLICATION_NON_UNIQUE);
 
-    setlocale(LC_ALL, "");
+    std::locale::global(std::locale(""));
     bindtextdomain(GETTEXT_PACKAGE, SNES9XLOCALEDIR);
     bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
     textdomain(GETTEXT_PACKAGE);
 
-    memset(&Settings, 0, sizeof(Settings));
+    struct sigaction sig_callback{};
+    sig_callback.sa_handler = S9xTerm;
+    sigaction(15, &sig_callback, NULL); // SIGTERM
+    sigaction(3, &sig_callback, NULL);  // SIGQUIT
+    sigaction(2, &sig_callback, NULL);  // SIGINT
+
+    Settings = {};
 
     // Original config fills out values this port doesn't.
     S9xLoadConfigFiles(argv, argc);
@@ -75,23 +82,25 @@ int main(int argc, char *argv[])
     settings->set_property("gtk-menu-images", gui_config->enable_icons);
     settings->set_property("gtk-button-images", gui_config->enable_icons);
 
-    S9xReportControllers();
-
     if (!Memory.Init() || !S9xInitAPU())
         exit(3);
 
     top_level = new Snes9xWindow(gui_config);
+#ifdef GDK_WINDOWING_X11
+    if (!GDK_IS_X11_WINDOW(top_level->window->get_window()->gobj()))
+        XInitThreads();
+#endif
 
     // Setting fullscreen before showing the window avoids some flicker.
     if ((gui_config->full_screen_on_open && rom_filename) || (gui_config->fullscreen))
         top_level->window->fullscreen();
-
     top_level->show();
 
     S9xInitDisplay(argc, argv);
 
     S9xPortSoundInit();
 
+    S9xReportControllers();
     for (int port = 0; port < 2; port++)
     {
         enum controllers type;
@@ -125,10 +134,6 @@ int main(int argc, char *argv[])
     top_level->update_accelerators();
 
     Settings.Paused = true;
-
-    Glib::signal_timeout().connect(sigc::ptr_fun(S9xPauseFunc), 100);
-    Glib::signal_timeout().connect(sigc::ptr_fun(S9xScreenSaverCheckFunc), 10000);
-
     S9xNoROMLoaded();
 
     if (rom_filename)
@@ -137,23 +142,18 @@ int main(int argc, char *argv[])
             top_level->window->unfullscreen();
     }
 
-    memset(&sig_callback, 0, sizeof(struct sigaction));
-    sig_callback.sa_handler = S9xTerm;
-
-    sigaction(15, &sig_callback, NULL); // SIGTERM
-    sigaction(3, &sig_callback, NULL);  // SIGQUIT
-    sigaction(2, &sig_callback, NULL);  // SIGINT
-
     // Perform the complete fullscreen process, including mode sets, which
     // didn't happen in the earlier Gtk call.
     if (gui_config->fullscreen)
         top_level->enter_fullscreen_mode();
 
-    gui_config->joysticks.flush_events();
-
     if (rom_filename && *Settings.InitialSnapshotFilename)
         S9xUnfreezeGame(Settings.InitialSnapshotFilename);
 
+    gui_config->joysticks.flush_events();
+
+    Glib::signal_timeout().connect(sigc::ptr_fun(S9xPauseFunc), 100);
+    Glib::signal_timeout().connect(sigc::ptr_fun(S9xScreenSaverCheckFunc), 10000);
     app->run(*top_level->window.get());
     return 0;
 }
@@ -181,43 +181,9 @@ int S9xOpenROM(const char *rom_filename)
 
     Settings.StopEmulation = !loaded;
 
-    if (!loaded && rom_filename)
-    {
-        char dir[_MAX_DIR + 1];
-        char drive[_MAX_DRIVE + 1];
-        char name[_MAX_FNAME + 1];
-        char ext[_MAX_EXT + 1];
-        char fname[_MAX_PATH + 1];
-
-        _splitpath(rom_filename, drive, dir, name, ext);
-        _makepath(fname, drive, dir, name, ext);
-
-        strcpy(fname, S9xGetDirectory(ROM_DIR));
-        strcat(fname, SLASH_STR);
-        strcat(fname, name);
-
-        if (ext[0])
-        {
-            strcat(fname, ".");
-            strcat(fname, ext);
-        }
-
-        _splitpath(fname, drive, dir, name, ext);
-        _makepath(fname, drive, dir, name, ext);
-
-        if ((Settings.StopEmulation = !Memory.LoadROM(fname)))
-        {
-            fprintf(stderr, _("Error opening: %s\n"), rom_filename);
-
-            loaded = false;
-        }
-        else
-            loaded = true;
-    }
-
     if (loaded)
     {
-        Memory.LoadSRAM(S9xGetFilename(".srm", SRAM_DIR));
+        Memory.LoadSRAM(S9xGetFilename(".srm", SRAM_DIR).c_str());
     }
     else
     {
@@ -234,7 +200,13 @@ int S9xOpenROM(const char *rom_filename)
 
     if (state_manager.init(gui_config->rewind_buffer_size * 1024 * 1024))
     {
-        printf("Using rewind buffer of %uMB\n", gui_config->rewind_buffer_size);
+        S9xMessage(
+            S9X_INFO,
+            S9X_NO_INFO,
+            fmt::format(_("Using rewind buffer of {0}\n"),
+                Glib::format_size(
+                    gui_config->rewind_buffer_size * 1024 * 1024,
+                    Glib::FORMAT_SIZE_IEC_UNITS).c_str()).c_str());
     }
 
     S9xROMLoaded();
@@ -260,12 +232,20 @@ void S9xNoROMLoaded()
 {
     S9xSoundStop();
     gui_config->rom_loaded = false;
-    S9xDisplayRefresh(-1, -1);
+    S9xDisplayRefresh();
     top_level->configure_widgets();
 }
 
 static bool S9xPauseFunc()
 {
+    static bool first_clear = false;
+
+    if (!first_clear)
+    {
+        top_level->refresh();
+        first_clear = true;
+    }
+
     S9xProcessEvents(true);
 
     if (!S9xNetplayPush())
@@ -325,6 +305,8 @@ static bool S9xIdleFunc()
             S9xNPSendPause(true);
         }
 
+        top_level->window->queue_draw();
+
         /* Move to a timer-based function to use less CPU */
         Glib::signal_timeout().connect(sigc::ptr_fun(S9xPauseFunc), 8);
         return false;
@@ -340,8 +322,6 @@ static bool S9xIdleFunc()
         return true;
     }
 
-    S9xThrottle(Settings.SkipFrames);
-
     if (!S9xNetplayPush())
     {
         if (Settings.Rewinding)
@@ -360,7 +340,7 @@ static bool S9xIdleFunc()
 
         if ((Settings.TurboMode || Settings.Rewinding) && gui_config->mute_sound_turbo)
             Settings.Mute |= 0x80;
-        else 
+        else
             Settings.Mute &= ~0x80;
 
         S9xMainLoop();
@@ -381,17 +361,60 @@ static bool S9xScreenSaverCheckFunc()
 
     return true;
 }
-
 /* Snes9x core hooks */
 void S9xMessage(int type, int number, const char *message)
 {
     switch (number)
     {
-    case S9X_MOVIE_INFO:
-        S9xSetInfoString(message);
-        break;
-    default:
-        break;
+        case S9X_MOVIE_INFO:
+        {
+            S9xSetInfoString(message);
+            break;
+        }
+        case S9X_ROM_INFO:
+        {
+            S9xSetInfoString(Memory.GetMultilineROMInfo().c_str());
+            break;
+        }
+        default:
+        {
+            switch (type)
+            {
+                case S9X_TRACE:
+                case S9X_DEBUG:
+                {
+                    g_debug(message);
+                    break;
+                }
+                case S9X_WARNING:
+                {
+                    g_warning(message);
+                    break;
+                }
+                case S9X_INFO:
+                {
+                    g_info(message);
+                    g_message(message);
+                    break;
+                }
+                case S9X_ERROR:
+                {
+                    // GLib’s g_critical() does not terminate the process
+                    g_critical(message);
+                    break;
+                }
+                case S9X_FATAL_ERROR:
+                {
+                    // GLib’s g_error() terminates the process
+                    g_error(message);
+                    break;
+                }
+                default:
+                {
+                    g_message(message);
+                }
+            }
+        }
     }
 }
 
@@ -400,64 +423,32 @@ void S9xParseArg(char **argv, int &i, int argc)
 {
     if (!strcasecmp(argv[i], "-filter"))
     {
+        std::string filter_names[] =
+        {
+            "none",
+            "supereagle",
+            "2xsai",
+            "super2xsai",
+            "epx",
+            "epx_smooth",
+            "ntsc",
+            "scanlines",
+            "simple2x",
+            "simple3x",
+            "simple4x",
+            "hq2x",
+            "hq3x",
+            "hq4x",
+            "2xbrz",
+            "3xbrz",
+            "4xbrz"
+        };
+
         if ((++i) < argc)
         {
-            if (!strcasecmp(argv[i], "none"))
-            {
-                gui_config->scale_method = FILTER_NONE;
-            }
-            else if (!strcasecmp(argv[i], "supereagle"))
-            {
-                gui_config->scale_method = FILTER_SUPEREAGLE;
-            }
-            else if (!strcasecmp(argv[i], "2xsai"))
-            {
-                gui_config->scale_method = FILTER_2XSAI;
-            }
-            else if (!strcasecmp(argv[i], "super2xsai"))
-            {
-                gui_config->scale_method = FILTER_SUPER2XSAI;
-            }
-#ifdef USE_HQ2X
-            else if (!strcasecmp(argv[i], "hq2x"))
-            {
-                gui_config->scale_method = FILTER_HQ2X;
-            }
-            else if (!strcasecmp(argv[i], "hq3x"))
-            {
-                gui_config->scale_method = FILTER_HQ3X;
-            }
-            else if (!strcasecmp(argv[i], "hq4x"))
-            {
-                gui_config->scale_method = FILTER_HQ4X;
-            }
-#endif /* USE_HQ2X */
-#ifdef USE_XBRZ
-            else if (!strcasecmp(argv[i], "2xbrz"))
-            {
-                gui_config->scale_method = FILTER_2XBRZ;
-            }
-            else if (!strcasecmp(argv[i], "3xbrz"))
-            {
-                gui_config->scale_method = FILTER_3XBRZ;
-            }
-            else if (!strcasecmp(argv[i], "4xbrz"))
-            {
-                gui_config->scale_method = FILTER_4XBRZ;
-            }
-#endif /* USE_XBRZ */
-            else if (!strcasecmp(argv[i], "epx"))
-            {
-                gui_config->scale_method = FILTER_EPX;
-            }
-            else if (!strcasecmp(argv[i], "ntsc"))
-            {
-                gui_config->scale_method = FILTER_NTSC;
-            }
-            else
-            {
-                gui_config->scale_method = FILTER_NONE;
-            }
+            for (int f = 0; f < NUM_FILTERS; f++)
+                if (filter_names[f] == argv[i])
+                    gui_config->scale_method = f;
         }
     }
     else if (!strcasecmp(argv[i], "-mutesound"))
@@ -520,6 +511,9 @@ static void S9xThrottle(int method)
     }
     else // THROTTLE_TIMER or THROTTLE_TIMER_FRAMESKIP
     {
+        if (S9xDisplayGetDriver()->can_throttle())
+            return;
+
         if (method == THROTTLE_TIMER_FRAMESKIP)
         {
             if (now - frame_clock > Settings.FrameTime)
@@ -553,6 +547,7 @@ static void S9xThrottle(int method)
 
 void S9xSyncSpeed()
 {
+    S9xThrottle(Settings.SkipFrames);
 }
 
 static void S9xCheckPointerTimer()
@@ -601,10 +596,11 @@ const char *S9xStringInput(const char *message)
 
 void S9xExtraUsage()
 {
-    printf("GTK port options:\n"
+    S9xMessage(S9X_INFO, S9X_USAGE,
+         _("GTK port options:\n"
            "-filter [option]               Use a filter to scale the image.\n"
            "                               [option] is one of: none supereagle 2xsai\n"
            "                               super2xsai hq2x hq3x hq4x 2xbrz 3xbrz 4xbrz epx ntsc\n"
            "\n"
-           "-mutesound                     Disables sound output.\n");
+           "-mutesound                     Disables sound output.\n"));
 }
