@@ -11,11 +11,13 @@
 #include "controls.h"
 #include "cheats.h"
 #include "movie.h"
-#include "logger.h"
 #include "display.h"
 #include "conffile.h"
 #include "crosshairs.h"
 #include <stdio.h>
+#include <vector>
+#include <string>
+
 #ifdef _WIN32
 #include <direct.h>
 #else
@@ -370,10 +372,6 @@ static void update_variables(void)
     var.value=NULL;
     Settings.Transparency=!(environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && !strcmp("disabled", var.value));
 
-    var.key="snes9x_gfx_hires";
-    var.value=NULL;
-    Settings.SupportHiRes=!(environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && !strcmp("disabled", var.value));
-
     var.key="snes9x_audio_interpolation";
     var.value=NULL;
     if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
@@ -751,26 +749,22 @@ static void update_variables(void)
     }
 }
 
-static void S9xAudioCallback(void*)
-{
-    const int BUFFER_SIZE = 256;
-    // This is called every time 128 to 132 samples are generated, which happens about 8 times per frame.  A buffer size of 256 samples is enough here.
-    static int16_t audio_buf[BUFFER_SIZE];
+void S9xSyncSpeed() {
+
+    if (Settings.Mute) {
+        S9xClearSamples();
+        return;
+    }
+
+    static std::vector<int16_t> audio_buffer;
 
     size_t avail = S9xGetSampleCount();
-    while (avail >= BUFFER_SIZE)
-    {
-        //this loop will never be entered, but handle oversized sample counts just in case
-        S9xMixSamples((uint8*)audio_buf, BUFFER_SIZE);
-        audio_batch_cb(audio_buf, BUFFER_SIZE >> 1);
 
-        avail -= BUFFER_SIZE;
-    }
-    if (avail > 0)
-    {
-        S9xMixSamples((uint8*)audio_buf, avail);
-        audio_batch_cb(audio_buf, avail >> 1);
-    }
+    if (audio_buffer.size() < avail)
+        audio_buffer.resize(avail);
+
+    S9xMixSamples((uint8*)&audio_buffer[0], avail);
+    audio_batch_cb(&audio_buffer[0], avail >> 1);
 }
 
 void retro_get_system_info(struct retro_system_info *info)
@@ -943,10 +937,10 @@ void retro_cheat_set(unsigned index, bool enabled, const char *codeline)
         }
 
         /* Goldfinger was broken and nobody noticed. Removed */
-        if (S9xAddCheatGroup ("retro", code) >= 0)
+        if (S9xAddCheatGroup (std::string("retro"), std::string(code)) >= 0)
         {
             if (enabled)
-                S9xEnableCheatGroup (Cheat.g.size () - 1);
+                S9xEnableCheatGroup (Cheat.group.size () - 1);
         }
         else
         {
@@ -1066,14 +1060,14 @@ static bool8 LoadBIOS(uint8 *biosrom, const char *biosname, int biossize)
     char	name[PATH_MAX + 1];
     bool8 r = FALSE;
 
-    strcpy(name, S9xGetDirectory(ROMFILENAME_DIR));
+    strcpy(name, S9xGetDirectory(ROMFILENAME_DIR).c_str());
     strcat(name, SLASH_STR);
     strcat(name, biosname);
 
     fp = fopen(name, "rb");
     if (!fp)
     {
-        strcpy(name, S9xGetDirectory(BIOS_DIR));
+        strcpy(name, S9xGetDirectory(BIOS_DIR).c_str());
         strcat(name, SLASH_STR);
         strcat(name, biosname);
 
@@ -1132,7 +1126,7 @@ bool retro_load_game(const struct retro_game_info *game)
         }
 
         else
-            rom_loaded = Memory.LoadROMMem((const uint8_t*)game->data ,game->size);
+            rom_loaded = Memory.LoadROMMem((const uint8_t*)game->data ,game->size, g_basename);
 
         if(biosrom) delete[] biosrom;
     }
@@ -1154,7 +1148,7 @@ bool retro_load_game(const struct retro_game_info *game)
         if (randomize_memory)
         {
             srand(time(NULL));
-            for(int lcv = 0; lcv < 0x20000; lcv++)
+            for(int lcv = 0; lcv < sizeof(Memory.RAM); lcv++)
                 Memory.RAM[lcv] = rand() % 256;
         }
     }
@@ -1346,7 +1340,6 @@ void retro_init(void)
     Settings.Stereo = TRUE;
     Settings.SoundPlaybackRate = 32040;
     Settings.SoundInputRate = 32040;
-    Settings.SupportHiRes = TRUE;
     Settings.Transparency = TRUE;
     Settings.AutoDisplayMessages = TRUE;
     Settings.InitialInfoStringTimeout = 120;
@@ -1370,16 +1363,13 @@ void retro_init(void)
         exit(1);
     }
 
-    S9xInitSound(0);
+    S9xInitSound(32);
 
     S9xSetSoundMute(FALSE);
-    S9xSetSamplesAvailableCallback(S9xAudioCallback, NULL);
+    S9xSetSamplesAvailableCallback(NULL, NULL);
 
-    GFX.Pitch = MAX_SNES_WIDTH_NTSC * sizeof(uint16);
-    screen_buffer = (uint16*) calloc(1, GFX.Pitch * (MAX_SNES_HEIGHT + 16));
-    GFX.Screen = screen_buffer + (GFX.Pitch >> 1) * 16;
-    ntsc_screen_buffer = (uint16*) calloc(1, GFX.Pitch * (MAX_SNES_HEIGHT + 16));
-    snes_ntsc_buffer = ntsc_screen_buffer + (GFX.Pitch >> 1) * 16;
+    ntsc_screen_buffer = (uint16*) calloc(1, MAX_SNES_WIDTH_NTSC * 2 * (MAX_SNES_HEIGHT + 16));
+    snes_ntsc_buffer = ntsc_screen_buffer + (MAX_SNES_WIDTH_NTSC >> 1) * 16;
     S9xGraphicsInit();
 
     S9xInitInputDevices();
@@ -1856,7 +1846,6 @@ void retro_run()
     poll_cb();
     report_buttons();
     S9xMainLoop();
-    S9xAudioCallback(NULL);
 }
 
 void retro_deinit()
@@ -2024,11 +2013,11 @@ bool8 S9xDeinitUpdate(int width, int height)
         burst_phase = (burst_phase + 1) % 3;
 
         if (width == 512)
-            snes_ntsc_blit_hires(snes_ntsc, GFX.Screen, GFX.Pitch / 2, burst_phase, width, height, snes_ntsc_buffer, GFX.Pitch);
+            snes_ntsc_blit_hires(snes_ntsc, GFX.Screen, GFX.Pitch / 2, burst_phase, width, height, snes_ntsc_buffer, MAX_SNES_WIDTH_NTSC * 2);
         else
-            snes_ntsc_blit(snes_ntsc, GFX.Screen, GFX.Pitch / 2, burst_phase, width, height, snes_ntsc_buffer, GFX.Pitch);
+            snes_ntsc_blit(snes_ntsc, GFX.Screen, GFX.Pitch / 2, burst_phase, width, height, snes_ntsc_buffer, MAX_SNES_WIDTH_NTSC * 2);
 
-        video_cb(snes_ntsc_buffer + ((int)(GFX.Pitch >> 1) * overscan_offset), SNES_NTSC_OUT_WIDTH(256), height, GFX.Pitch);
+        video_cb(snes_ntsc_buffer + ((int)(MAX_SNES_WIDTH_NTSC) * overscan_offset), SNES_NTSC_OUT_WIDTH(256), height, MAX_SNES_WIDTH_NTSC * 2);
     }
     else if (width == MAX_SNES_WIDTH && hires_blend)
     {
@@ -2092,7 +2081,6 @@ bool8 S9xContinueUpdate(int width, int height)
 
 // Dummy functions that should probably be implemented correctly later.
 void S9xParsePortConfig(ConfigFile&, int) {}
-void S9xSyncSpeed() {}
 const char* S9xStringInput(const char* in) { return in; }
 
 #ifdef _WIN32
@@ -2101,41 +2089,23 @@ const char* S9xStringInput(const char* in) { return in; }
 #define SLASH '/'
 #endif
 
-const char* S9xGetFilename(const char* in, s9x_getdirtype type)
-{
-    static char newpath[2048];
-
-    newpath[0] = '\0';
-
-    switch (type)
-    {
-        case ROMFILENAME_DIR:
-            sprintf(newpath, "%s%c%s%s", g_rom_dir, SLASH, g_basename, in);
-            return newpath;
-        default:
-            break;
-    }
-
-    return in;
-}
-
-const char* S9xGetDirectory(s9x_getdirtype type)
+std::string S9xGetDirectory(s9x_getdirtype type)
 {
     switch (type)
     {
         case BIOS_DIR:
-            return retro_system_directory;
+            return std::string(retro_system_directory);
         default:
-            return g_rom_dir;
+            return std::string(g_rom_dir);
     }
 
-    return "";
+    return std::string("");
 }
 void S9xInitInputDevices() {}
 void S9xHandlePortCommand(s9xcommand_t, short, short) {}
 bool S9xPollButton(unsigned int, bool*) { return false; }
 void S9xToggleSoundChannel(int) {}
-const char* S9xGetFilenameInc(const char* in, s9x_getdirtype) { return ""; }
+std::string S9xGetFilenameInc(std::string in, s9x_getdirtype) { return ""; }
 const char* S9xBasename(const char* in) { return in; }
 bool8 S9xInitUpdate() { return TRUE; }
 void S9xExtraUsage() {}
