@@ -94,7 +94,7 @@ static int CyclesUntilNext (int hc, int vc)
 		// Add number of lines
 		total += (vc - vpos) * Timings.H_Max_Master;
 		// If line 240 is in there and we're odd, subtract a dot
-		if (vpos <= 240 && vc > 240 && Timings.InterlaceField & !IPPU.Interlace)
+		if (vpos <= 240 && vc > 240 && S9xInterlaceField() & !IPPU.Interlace)
 			total -= ONE_DOT_CYCLE;
 	}
 	else
@@ -105,11 +105,11 @@ static int CyclesUntilNext (int hc, int vc)
 		}
 
 		total += (Timings.V_Max - vpos) * Timings.H_Max_Master;
-		if (vpos <= 240 && Timings.InterlaceField && !IPPU.Interlace)
+		if (vpos <= 240 && S9xInterlaceField() && !IPPU.Interlace)
 			total -= ONE_DOT_CYCLE;
 
 		total += (vc) * Timings.H_Max_Master;
-		if (vc > 240 && !Timings.InterlaceField && !IPPU.Interlace)
+		if (vc > 240 && !S9xInterlaceField() && !IPPU.Interlace)
 			total -= ONE_DOT_CYCLE;
 	}
 
@@ -146,7 +146,7 @@ void S9xUpdateIRQPositions (bool initial)
 		}
 
 		// Check for short dot scanline
-		if (v_pos == 240 && Timings.InterlaceField && !IPPU.Interlace)
+		if (v_pos == 240 && S9xInterlaceField() && !IPPU.Interlace)
 		{
 			Timings.NextIRQTimer -= PPU.IRQHBeamPos <= 322 ? ONE_DOT_CYCLE / 2 : 0;
 			Timings.NextIRQTimer -= PPU.IRQHBeamPos <= 326 ? ONE_DOT_CYCLE / 2 : 0;
@@ -164,7 +164,7 @@ void S9xUpdateIRQPositions (bool initial)
 		Timings.NextIRQTimer = CyclesUntilNext (PPU.HTimerPosition, PPU.VTimerPosition);
 
 		// Check for short dot scanline
-		int field = Timings.InterlaceField;
+		int field = S9xInterlaceField();
 
 		if (PPU.VTimerPosition < CPU.V_Counter ||
 		   (PPU.VTimerPosition == CPU.V_Counter && Timings.NextIRQTimer > Timings.H_Max))
@@ -1894,7 +1894,6 @@ void S9xSoftResetPPU (void)
 	memset(IPPU.TileCached[TILE_4BIT_EVEN], 0, MAX_4BIT_TILES);
 	memset(IPPU.TileCached[TILE_4BIT_ODD], 0,  MAX_4BIT_TILES);
 	PPU.VRAMReadBuffer = 0; // XXX: FIXME: anything better?
-	GFX.InterlaceFrame = 0;
 	GFX.DoInterlace = 0;
 	IPPU.Interlace = FALSE;
 	IPPU.InterlaceOBJ = FALSE;
