@@ -228,7 +228,7 @@ static bool8 S9xNPSSendData (int fd, const uint8 *data, int length)
 void S9xNPSendHeartBeat ()
 {
     int len = 3;
-    uint8 data [3 + 4 * 5];
+    uint8 data [(3 + 4) * 8];
     uint8 *ptr = data;
     int n;
 
@@ -960,7 +960,7 @@ bool8 S9xNPSendROMImageToClient (int c)
     uint8 header [7 + 1 + 4];
     uint8 *ptr = header;
     int len = sizeof (header) + Memory.CalculatedSize +
-              strlen (Memory.ROMFilename) + 1;
+              Memory.ROMFilename.length() + 1;
     *ptr++ = NP_SERV_MAGIC;
     *ptr++ = NPServer.Clients [c].SendSequenceNum++;
     *ptr++ = NP_SERV_ROM_IMAGE;
@@ -972,8 +972,8 @@ bool8 S9xNPSendROMImageToClient (int c)
     if (!S9xNPSSendData (NPServer.Clients [c].Socket, header, sizeof (header)) ||
         !S9xNPSSendData (NPServer.Clients [c].Socket, Memory.ROM,
                         Memory.CalculatedSize) ||
-        !S9xNPSSendData (NPServer.Clients [c].Socket, (uint8 *) Memory.ROMFilename,
-                        strlen (Memory.ROMFilename) + 1))
+        !S9xNPSSendData (NPServer.Clients [c].Socket, (uint8 *) Memory.ROMFilename.c_str(),
+                        Memory.ROMFilename.length() + 1))
     {
         S9xNPShutdownClient (c, TRUE);
         return (FALSE);
@@ -1031,7 +1031,7 @@ void S9xNPSyncClient (int client)
                 S9xNPRecomputePause ();
                 S9xNPSendFreezeFile (client, data, len);
             }
-            delete data;
+            delete[] data;
         }
         remove (fname);
     }
@@ -1208,7 +1208,7 @@ void S9xNPSendFreezeFileToAllClients (const char *filename)
             if (NPServer.Clients [c].SaidHello)
                 S9xNPSendFreezeFile (c, data, len);
         }
-        delete data;
+        delete[] data;
     }
 }
 
