@@ -48,7 +48,6 @@
 #include "ppu.h"
 #include "controls.h"
 #include "movie.h"
-#include "logger.h"
 #include "conffile.h"
 #include "blit.h"
 #include "display.h"
@@ -92,7 +91,6 @@ struct GUIData
 	uint32			blue_size;
 	Window			window;
 	Image			*image;
-	uint8			*snes_buffer;
 	uint8			*filter_buffer;
 	uint8			*blit_screen;
 	uint32			blit_screen_pitch;
@@ -624,7 +622,7 @@ static bool8 SetupXvideo()
 		printf("Selected XvRGB format: %d bpp\n",GUI.xv_bpp);
 	} else {
 		// use I420 or YUY2
-		if(GUI.xv_format == FOURCC_I420) { 
+		if(GUI.xv_format == FOURCC_I420) {
 		    printf("Xvideo I420 image format.\n");
         } else {
 		    printf("Xvideo YUY2 image format.\n");
@@ -932,14 +930,6 @@ static void SetupImage (void)
 #endif
 		SetupXImage();
 
-	// Setup SNES buffers
-	GFX.Pitch = SNES_WIDTH * 2 * 2;
-	GUI.snes_buffer = (uint8 *) calloc(GFX.Pitch * ((SNES_HEIGHT_EXTENDED + 4) * 2), 1);
-	if (!GUI.snes_buffer)
-		FatalError("Failed to allocate GUI.snes_buffer.");
-
-	GFX.Screen = (uint16 *) (GUI.snes_buffer + (GFX.Pitch * 2 * 2));
-
 	GUI.filter_buffer = (uint8 *) calloc((SNES_WIDTH * 2) * 2 * (SNES_HEIGHT_EXTENDED * 2), 1);
 	if (!GUI.filter_buffer)
 		FatalError("Failed to allocate GUI.filter_buffer.");
@@ -967,12 +957,6 @@ static void SetupImage (void)
 
 static void TakedownImage (void)
 {
-	if (GUI.snes_buffer)
-	{
-		free(GUI.snes_buffer);
-		GUI.snes_buffer = NULL;
-	}
-
 	if (GUI.filter_buffer)
 	{
 		free(GUI.filter_buffer);
@@ -1561,9 +1545,6 @@ static void Repaint (bool8 isFrameBoundry)
 			XDefineCursor(GUI.display, GUI.window, GUI.point_cursor);
 		}
 	}
-
-	if (Settings.DumpStreams && isFrameBoundry)
-		S9xVideoLogger(GUI.image->data, SNES_WIDTH * 2, SNES_HEIGHT_EXTENDED * 2, GUI.bytes_per_pixel, GUI.image->bytes_per_line);
 }
 
 void S9xTextMode (void)
@@ -1650,40 +1631,6 @@ void S9xProcessEvents (bool8 block)
 
 const char * S9xSelectFilename (const char *def, const char *dir1, const char *ext1, const char *title)
 {
-	static char	s[PATH_MAX + 1];
-	char		buffer[PATH_MAX + 1];
-
-	SetXRepeat(TRUE);
-
-	printf("\n%s (default: %s): ", title, def);
-	fflush(stdout);
-
-	SetXRepeat(FALSE);
-
-	if (fgets(buffer, PATH_MAX + 1, stdin))
-	{
-		char	drive[_MAX_DRIVE + 1], dir[_MAX_DIR + 1], fname[_MAX_FNAME + 1], ext[_MAX_EXT + 1];
-
-		char	*p = buffer;
-		while (isspace(*p))
-			p++;
-		if (!*p)
-		{
-			strncpy(buffer, def, PATH_MAX + 1);
-			buffer[PATH_MAX] = 0;
-			p = buffer;
-		}
-
-		char	*q = strrchr(p, '\n');
-		if (q)
-			*q = 0;
-
-		_splitpath(p, drive, dir, fname, ext);
-		_makepath(s, drive, *dir ? dir : dir1, fname, *ext ? ext : ext1);
-
-		return (s);
-	}
-
 	return (NULL);
 }
 
@@ -1871,4 +1818,4 @@ bool S9xDisplayPollPointer (uint32 id, int16 *x, int16 *y)
 	*y = GUI.mouse_y;
 
 	return (true);
-}
\ No newline at end of file
+}
